"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.codeMutex = exports.concurrency = exports.go = void 0;
const tslib_1 = require("tslib");
const go = (code) => { code().catch(() => { }); };
exports.go = go;
class Task {
    constructor(code) {
        this.code = code;
        this.promise = new Promise((resolve, reject) => {
            this.resolve = resolve;
            this.reject = reject;
        });
    }
}
const concurrency = (limit) => {
    let workers = 0;
    const queue = new Set();
    const work = () => (0, tslib_1.__awaiter)(void 0, void 0, void 0, function* () {
        const task = queue.values().next().value;
        if (task)
            queue.delete(task);
        else
            return;
        workers++;
        try {
            task.resolve(yield task.code());
        }
        catch (error) {
            task.reject(error);
        }
        finally {
            workers--, queue.size && (0, exports.go)(work);
        }
    });
    return (code) => (0, tslib_1.__awaiter)(void 0, void 0, void 0, function* () {
        const task = new Task(code);
        queue.add(task);
        return workers < limit && (0, exports.go)(work), task.promise;
    });
};
exports.concurrency = concurrency;
const codeMutex = (code) => {
    let result;
    return () => (0, tslib_1.__awaiter)(void 0, void 0, void 0, function* () {
        if (result)
            return result;
        try {
            return yield (result = code());
        }
        finally {
            result = undefined;
        }
    });
};
exports.codeMutex = codeMutex;
